/**
 * @anue-package Anue.Auth
 * @doc
 * Auth模組負責管理Token的交換、管理、刷新等環節。目前的架構上，由於使用者痛
 * 一使用[Cognito](https://aws.amazon.com/tw/cognito/)進行統一的身份識別因此不論是透過第三方的身份或是我們自己的
 * 認證機制（如Email）最後都得再透過API來取得Cognito ID和authorization token。
 *
 * ## 範例 (Social與Email登入)
 *
 * ```Javascript
 * import 'anue-fe-sdk'
 * import 'anue-fe-sdk/web-preset'
 * import Auth from 'anue-fe-sdk/Auth'
 * import GoogleWebProvider from 'anue-fe-sdk/GoogleWebProvider'
 * import FBWebProvider from 'anue-fe-sdk/FBWebProvider'
 * import EmailProvider from 'anue-fe-sdk/EmailProvider'
 * import Constants from 'anue-fe-sdk/Constants
 *
 * const google = new GoogleWebProvider({ apiKey, clientId })
 * const fb = new FBWebProvider({ clientId })
 * const email = new EmailProvider()
 *
 *
 * const auth = new Auth()
 *  // optional
 *  .channel(Constants.SupportedChannel.Stock)
 *  .onStateChange(ctx => {
 *    console.log('Auth state changed', ctx)
 *  })
 *
 * auth.connectProvider(google, fb, email)
 *
 *
 * ```
 */
import AbstractProvider from '@packages/Provider/AbstractProvider.abstract';
import { SupportedChannel } from '@constants/Keys';
import Eventy, { EventyHandler } from '@utils/eventy';
declare class Auth {
    private mHost;
    private mChannel;
    private mIdToken;
    private mAccessToken;
    private mCognitoId;
    private mAuthorization;
    private mIsNewComer;
    private removeRequestInspector;
    private removeResponseInspector;
    private mConnectedProviders;
    private mIsReady;
    private mPrimaryProvider;
    private mProfile;
    private mSSOProvider;
    private mPortalUrl;
    private mPostLoginProvider;
    private mSyncLock;
    event: Eventy;
    constructor(config?: Anue.Auth.Config);
    host: (host: string) => Auth;
    channel: (channel: SupportedChannel) => Auth;
    portal: (url: string) => Auth;
    launchPortal: () => Promise<void>;
    ready: () => Promise<Anue.Auth.Credentials | null>;
    getPrimaryProvider: () => AbstractProvider | null;
    /**
     * @doc
     * @method onStateChange
     * @param fn EventyHandler<Anue.Auth.StateChangeEvent> 處理Auth狀態改變的函數，接受的參數是一個[Anue.Auth.StateChangeEvent]()物件
     * @returns [Auth](->Anue.Auth)
     * @public
     * 透過這個方法可以加入一個函數在Auth的狀態改變時被觸發，這也是應用端和Auth互動最主要的一個API。
     */
    onStateChange: (fn: EventyHandler<Anue.Auth.StateChangeEvent>) => Auth;
    onError: (fn: EventyHandler<Anue.Auth.ErrorChangeEvent>) => this;
    private consumeState;
    setSSOProvider: (name: string) => Promise<this>;
    /**
     * @doc
     * @private
     * @method requestNetworkInspector
     * >! 這個方法不為外部所引用
     * 這個方法會在認證步驟`renew`或者`consume`完成之後自動由`Auth`添加至`Network`的
     * `requestInspector`佇列之中，此後每當有任何請求時`authorization`會被自動加入在
     * 請求的header中。
     */
    private requestNetworkInspector;
    /**
     * @doc
     * @private
     * @method responseNetworkInspector
     * >! 這個方法不為外部所引用
     * 這個方法會在認證步驟`renew`或者`consume`完成之後自動由`Auth`添加至`Network`的
     * `responseInspector`佇列之中，之後當請求回應狀態代碼為`401`時，這個方法會自動呼叫
     * [Auth.renew](#renew)並且試著將原本的請求**重送**。
     */
    private responseNetworkInspector;
    /**
     * @doc
     * @method getInitialURL
     * @private
     * 這個方法會依照當前的環境返回預設的登入驗證伺服器URL，在一般瀏覽器的環境下
     * 會返回當前的`hostname`作為預設的host來發出登入等相關的API
     * 在`React Native`或者是`NodeJS`這類的環境下預設會產生一個**空字串**
     */
    private static getInitialURL;
    /**
     * @doc
     * @public
     * @promise
     * @method renew
     * @param provider? 一個Provider的實體
     * @returns Anue.Auth.Credentials | null
     * 使用目前所儲存的使用者資訊來刷新`authorization` (呼叫我們自己提供的API時使用的token)
     * 對於`Email`來說這個行為等同於`登入`。
     */
    renew: (ref?: any) => Promise<Anue.Auth.Credentials | null>;
    private renewRequest;
    private initializeProviders;
    lazyConnectProvider: (...providers: AbstractProvider[]) => Auth;
    /**
     * @doc
     * @name connectProvider
     * @param provider AbstractProvider 一個[Provider](->Anue.AbstractProvider)物件
     * 將這個provider物件的狀態同步連結到當前的Auth實體，這個行為會將[auth.consume](#consume)與該
     * provider之[login](->Anue.AbstractProvder#login)與[logout](->Anue.AbstractProvder#logout)方法進行連結。
     */
    connectProvider: (...providers: AbstractProvider[]) => Auth;
    private setCredentials;
    private syncProviderToken;
    private rpcCall;
    /**
     * @doc
     * @method clearCredentials
     * 清除所有儲存的認證資訊，等同於`logout`。當這個Auth物件有使用[connectProvider](->Anue.Auth#connectProvider)
     * 連結其他[Provider](->Anue.AbstractProvider)這個方法會在連結的Provider呼叫`logout`時一併被呼叫。
     */
    clearCredentials: () => Promise<void>;
    logout: () => Promise<void>;
    /**
     * Get last stored credential context object
     */
    getCredentialContext: () => Anue.Auth.Credentials | null;
}
export default Auth;
